@@ -37,10 +37,12 @@
 #include <glade/glade.h>
 
 #define SELECT_RESULT_FONT _("Select result font")
+#define SELECT_STACK_FONT _("Select stack font")
 #define SELECT_MODULE_FONT _("Select module font")
 #define SELECT_ACT_MOD_COLOR _("Select active module color")
 #define SELECT_INACT_MOD_COLOR _("Select inactive module color")
 #define SELECT_RESULT_FONT_COLOR _("Select result font color")
+#define SELECT_STACK_COLOR _("Select stack color")
 #define SELECT_BKG_COLOR _("Select background color")
 #define SELECT_BUTTON_FONT _("Select button font")	
 
@@ -52,14 +54,30 @@ void
 on_quit_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	// remember display's value
-	if (prefs.rem_value) g_free (prefs.rem_value);
-	prefs.rem_value = display_result_get();
+	char 	**stack;
+	
+	/* remember some things */
 	if (prefs.mode == SCIENTIFIC_MODE) {
-		// save number and angle mode only in scientific mode.
+		/* save number and angle mode only in scientific mode. */
 		prefs.def_number = current_status.number;
 		prefs.def_angle = current_status.angle;
 	}
+	if (prefs.rem_valuex) g_free (prefs.rem_valuex);
+	prefs.rem_valuex = display_result_get();
+	if (current_status.notation == CS_RPN) {
+		if (prefs.rem_valuey) g_free (prefs.rem_valuey);
+		if (prefs.rem_valuez) g_free (prefs.rem_valuez);
+		if (prefs.rem_valuet) g_free (prefs.rem_valuet);
+		/* we only save the visible stack */
+		stack = display_stack_get_yzt();
+		prefs.rem_valuey = g_strdup(stack[0]);
+		g_free (stack[0]);
+		prefs.rem_valuez = g_strdup(stack[1]);
+		g_free (stack[1]);
+		prefs.rem_valuet = g_strdup(stack[2]);
+		g_free (stack[2]);
+		g_free (stack);
+	}
 	prefs.def_notation = current_status.notation;
 	gtk_main_quit();
 }
@@ -117,7 +135,7 @@ on_operation_button_clicked            (GtkToggleButton       *button,
                                         gpointer         user_data)
 {
 	s_cb_token		current_token;
-	double			return_value;
+	double			return_value, *stack;
 	GtkWidget		*tbutton;
 	
 	if (gtk_toggle_button_get_active(button) == FALSE) return;
@@ -127,7 +145,7 @@ on_operation_button_clicked            (GtkToggleButton       *button,
 	if (current_token.operation != '(') 
 		current_token.number.value = display_result_get_double ();
 	else current_token.number.func = NULL;	
-	// do inverse left shift is a right shift
+	/* do inverse left shift is a right shift */
 	if ((current_token.operation == '<') && \
 		(BIT (current_status.fmod, CS_FMOD_FLAG_INV) == 1)) {
 			tbutton = glade_xml_get_widget (button_box_xml, "button_inv");
@@ -172,10 +190,18 @@ on_operation_button_clicked            (GtkToggleButton       *button,
 		switch (current_token.operation) {
 		case '=':
 			rpn_stack_push (current_token.number.value);
+			stack = rpn_stack_get (RPN_FINITE_STACK);
+			display_stack_set_yzt_double (stack);
+			free (stack);
 			current_status.rpn_have_result = FALSE;
+			/* display line isn't cleared! */
 			break;
 		default:
+			current_status.rpn_have_result = FALSE;
 			display_result_set_double (rpn_stack_operation (current_token));
+			stack = rpn_stack_get (RPN_FINITE_STACK);
+			display_stack_set_yzt_double (stack);
+			free (stack);
 			current_status.rpn_have_result = TRUE;
 		}
 	} else error_message ("on_operation_button_clicked: unknown status");
@@ -183,9 +209,9 @@ on_operation_button_clicked            (GtkToggleButton       *button,
 	current_status.calc_entry_start_new = TRUE;
 }
 
-/* this callback is called if a button for a function manipulating the current entry directly
- * is clicked. the array function_list knows the relation between button label and function to
- * call.
+/* this callback is called if a button for a function manipulating the current 
+ * entry directly is clicked. the array function_list knows the relation between 
+ * button label and function to call.
  */
 
 void
@@ -232,7 +258,10 @@ void constants_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 	char		*const_value;
 	
 	const_value = user_data;
+	current_status.rpn_have_result = TRUE;
 	display_result_set (const_value);
+	current_status.rpn_have_result = TRUE;
+	current_status.calc_entry_start_new = TRUE;
 }
 
 
@@ -273,7 +302,7 @@ on_gfunc_button_clicked                (GtkToggleButton       *button,
 	if (gtk_toggle_button_get_active(button) == FALSE) return;
 	button_activation (button);
 	func = g_object_get_data (G_OBJECT (button), "func");
-	if (func) func();
+	if (func) func(button);
 	else error_message ("This button has no function associated with");
 }
 
@@ -286,7 +315,7 @@ on_dec_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	//printf ("dec\n");
+	/*printf ("dec\n");*/
 	display_change_option (CS_DEC, DISPLAY_OPT_NUMBER);
 }
 
@@ -296,7 +325,7 @@ on_hex_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	//printf ("hex\n");
+	/*printf ("hex\n");*/
 	display_change_option (CS_HEX, DISPLAY_OPT_NUMBER);
 }
 
@@ -306,7 +335,7 @@ on_oct_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	//printf ("oct\n");
+	/*printf ("oct\n");*/
 	display_change_option (CS_OCT, DISPLAY_OPT_NUMBER);
 }
 
@@ -316,7 +345,7 @@ on_bin_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	//printf ("bin\n");
+	/*printf ("bin\n");*/
 	display_change_option (CS_BIN, DISPLAY_OPT_NUMBER);
 }
 
@@ -326,7 +355,7 @@ on_deg_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	//printf ("deg\n");
+	/*printf ("deg\n");*/
 	display_change_option (CS_DEG, DISPLAY_OPT_ANGLE);
 }
 
@@ -336,7 +365,7 @@ on_rad_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	//printf ("rad\n");
+	/*printf ("rad\n");*/
 	display_change_option (CS_RAD, DISPLAY_OPT_ANGLE);
 }
 
@@ -346,7 +375,7 @@ on_grad_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	if (!gtk_check_menu_item_get_active((GtkCheckMenuItem *)menuitem)) return;
-	//printf ("grad\n");
+	/*printf ("grad\n");*/
 	display_change_option (CS_GRAD, DISPLAY_OPT_ANGLE);
 }
 
@@ -358,7 +387,7 @@ on_ordinary_activate                  (GtkMenuItem     *menuitem,
 	GtkWidget	*w;
 	
 	if (((GtkCheckMenuItem *)menuitem)->active == FALSE) return;
-	//printf ("pan\n");
+	/* printf ("pan\n"); */
 	display_change_option (CS_PAN, DISPLAY_OPT_NOTATION);
 	rpn_free ();
 	all_clear ();
@@ -366,6 +395,14 @@ on_ordinary_activate                  (GtkMenuItem     *menuitem,
 	if (w) gtk_button_set_label ((GtkButton *)w, _("="));
 	w = glade_xml_get_widget (button_box_xml, "button_pow");
 	if (w) gtk_button_set_label ((GtkButton *)w, _("x^y"));
+	w = glade_xml_get_widget (button_box_xml, "button_f1");
+	if (w) gtk_button_set_label ((GtkButton *)w, _("("));
+	w = glade_xml_get_widget (button_box_xml, "button_f2");
+	if (w) gtk_button_set_label ((GtkButton *)w, _(")"));
+	display_stack_remove();
+	update_dispctrl();
+	/* pixel above/below display result line */
+	display_update_tags ();
 }
 
 
@@ -376,7 +413,6 @@ on_rpn_activate                       (GtkMenuItem     *menuitem,
 	GtkWidget	*w;
 	
 	if (((GtkCheckMenuItem *)menuitem)->active == FALSE) return;
-	//printf ("rpn\n");
 	display_change_option (CS_RPN, DISPLAY_OPT_NOTATION);
 	alg_free ();
 	all_clear ();
@@ -384,6 +420,14 @@ on_rpn_activate                       (GtkMenuItem     *menuitem,
 	if (w) gtk_button_set_label ((GtkButton *)w, _("ENT"));
 	w = glade_xml_get_widget (button_box_xml, "button_pow");
 	if (w) gtk_button_set_label ((GtkButton *)w, _("y^x"));
+	w = glade_xml_get_widget (button_box_xml, "button_f1");
+	if (w) gtk_button_set_label ((GtkButton *)w, _("x<>y"));
+	w = glade_xml_get_widget (button_box_xml, "button_f2");
+	if (w) gtk_button_set_label ((GtkButton *)w, _("roll"));
+	/* stack is created by all_clear */
+	update_dispctrl();
+	/* pixel above/below display result line */
+	display_update_tags ();
 }
 
 void
@@ -438,7 +482,7 @@ on_basic_mode_activate (GtkMenuItem     *menuitem,
 	
 	if (((GtkCheckMenuItem *) menuitem)->active == FALSE) return;
 	if (prefs.mode == SCIENTIFIC_MODE) {
-		// remember number and angle. notation is active in basic mode
+		/* remember number and angle. notation is active in basic mode */
 		prefs.def_number = current_status.number;
 		prefs.def_angle = current_status.angle;
 	}
@@ -446,11 +490,17 @@ on_basic_mode_activate (GtkMenuItem     *menuitem,
 	
 	ui_main_window_buttons_destroy ();
 	ui_main_window_buttons_create (prefs.mode);
-	ui_main_window_set_dispctrl (DISPCTRL_BOTTOM);
-
-	display_update_modules();
+	update_dispctrl();
 	
-	activate_menu_item ("dec");
+	display_update_modules();
+
+	/* In basic mode:
+	 *	- number base is always decimal.
+	 *	- ignore angle, as there are no angle operations in basic mode.
+	 *	- notation is fully functional.
+	 */	
+	display_module_number_activate (CS_DEC);
+	display_module_notation_activate (current_status.notation);
 	
 	menu_item = glade_xml_get_widget (main_window_xml, "display_control");
 	if (((GtkCheckMenuItem *) menu_item)->active == prefs.vis_dispctrl)
@@ -481,13 +531,14 @@ on_scientific_mode_activate (GtkMenuItem *menuitem,
 
 	ui_main_window_buttons_destroy ();
 	ui_main_window_buttons_create (prefs.mode);
-	ui_main_window_set_dispctrl (DISPCTRL_RIGHT);
-	
+
 	display_update_modules();
 	display_module_number_activate (prefs.def_number);
 	display_module_angle_activate (prefs.def_angle);
+	display_module_notation_activate (current_status.notation);
 
 	update_active_buttons (current_status.number, current_status.notation);
+	update_dispctrl();
 	
 	menu_item = glade_xml_get_widget (main_window_xml, "functions");
 	gtk_widget_set_sensitive (menu_item, TRUE);
@@ -594,7 +645,6 @@ on_prefs_result_font_clicked           (GtkButton       *button,
 	font_dialog = ui_font_dialog_create (SELECT_RESULT_FONT, button);
 }
 
-
 void
 on_prefs_result_color_clicked          (GtkButton       *button,
                                         gpointer         user_data)
@@ -602,6 +652,19 @@ on_prefs_result_color_clicked          (GtkButton       *button,
 	color_dialog = ui_color_dialog_create (SELECT_RESULT_FONT_COLOR, button);
 }
 
+void
+on_prefs_stack_font_clicked           (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	font_dialog = ui_font_dialog_create (SELECT_STACK_FONT, button);
+}
+
+void
+on_prefs_stack_color_clicked          (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	color_dialog = ui_color_dialog_create (SELECT_STACK_COLOR, button);
+}
 
 void
 on_prefs_mod_font_clicked              (GtkButton       *button,
@@ -668,6 +731,13 @@ on_color_ok_button_clicked             (GtkButton       *button,
 		prefs.result_color = gdk_color_to_string(color);
 		display_update_tags();
 	}
+	else if (strcmp (title, SELECT_STACK_COLOR) == 0)
+	{
+		da = glade_xml_get_widget (prefs_xml, "prefs_stack_color");
+		if (prefs.stack_color != NULL) g_free (prefs.stack_color);
+		prefs.stack_color = gdk_color_to_string(color);
+		display_update_tags();
+	}
 	else if (strcmp (title, SELECT_ACT_MOD_COLOR) == 0)
 	{
 		da = glade_xml_get_widget (prefs_xml, "prefs_act_mod_color");
@@ -709,6 +779,13 @@ on_font_ok_button_clicked              (GtkButton       *button,
 		prefs.result_font = g_strdup(font_name);
 		display_update_tags();
 	}
+	else if (strcmp (title, SELECT_STACK_FONT) == 0)
+	{
+		font_button = (GtkButton *) glade_xml_get_widget (prefs_xml, "prefs_stack_font");
+		if (prefs.stack_font != NULL) g_free (prefs.stack_font);
+		prefs.stack_font = g_strdup(font_name);
+		display_update_tags();
+	}
 	else if (strcmp (title, SELECT_MODULE_FONT) == 0)
 	{
 		font_button = (GtkButton *) glade_xml_get_widget (prefs_xml, "prefs_mod_font");
@@ -839,7 +916,7 @@ void on_prefs_rem_display_toggled (GtkToggleButton *togglebutton,
 				gpointer user_data)
 {
 	prefs.rem_display = gtk_toggle_button_get_active (togglebutton);
-	// only is important when leaving galculator
+	/* only is important when leaving galculator */
 }
 /*
 void on_prefs_def_number_changed (GtkOptionMenu *optionmenu,
@@ -887,6 +964,7 @@ void ms_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 		memory.len++;
 	}
 	memory.data[index] = display_result_get_double();
+	current_status.calc_entry_start_new = TRUE;
 }
 
 void
@@ -908,6 +986,8 @@ void mr_menu_handler (GtkMenuItem *menuitem, gpointer user_data)
 	
 	index = GPOINTER_TO_INT(user_data);
 	display_result_set_double(memory.data[index]);
+	current_status.rpn_have_result = TRUE;
+	current_status.calc_entry_start_new = TRUE;
 }
 
 void
@@ -1107,11 +1187,8 @@ void on_prefs_cupdate_clicked (GtkButton *button, gpointer user_data)
 void on_prefs_hex_bits_value_changed (GtkSpinButton *spinbutton,
 					GtkScrollType arg1,
 					gpointer user_data)
-{
-	extern int 	display_lengths[NR_NUMBER_BASES];
-	
+{	
 	prefs.hex_bits = (int) gtk_spin_button_get_value (spinbutton);
-	display_lengths[CS_HEX] = prefs.hex_bits/4;
 }
 
 void on_prefs_hex_signed_toggled (GtkToggleButton *togglebutton, 
@@ -1124,10 +1201,7 @@ void on_prefs_oct_bits_value_changed (GtkSpinButton *spinbutton,
 					GtkScrollType arg1,
 					gpointer user_data)
 {
-	extern int 	display_lengths[NR_NUMBER_BASES];
-	
 	prefs.oct_bits = (int) gtk_spin_button_get_value (spinbutton);
-	display_lengths[CS_OCT] = prefs.oct_bits/3;
 }
 
 void on_prefs_oct_signed_toggled (GtkToggleButton *togglebutton, 
@@ -1140,10 +1214,7 @@ void on_prefs_bin_bits_value_changed (GtkSpinButton *spinbutton,
 					GtkScrollType arg1,
 					gpointer user_data)
 {
-	extern int 	display_lengths[NR_NUMBER_BASES];
-	
 	prefs.bin_bits = (int) gtk_spin_button_get_value (spinbutton);
-	display_lengths[CS_BIN] = prefs.bin_bits/1;
 }
 
 
@@ -1181,7 +1252,7 @@ void on_main_window_check_resize (GtkContainer *container,
 {
 	static gboolean		itsme=FALSE;
 	
-	// is there a nicer way to to this?
+	/* is there a nicer way to to this? */
 	if (itsme) {
 		itsme = FALSE;
 		return;
@@ -1190,5 +1261,15 @@ void on_main_window_check_resize (GtkContainer *container,
 	itsme = TRUE;
 }
 
+void on_finite_stack_size_clicked (GtkRadioButton *rb, gpointer user_data)
+{
+	prefs.stack_size = RPN_FINITE_STACK;
+	rpn_stack_set_size (prefs.stack_size);
+}
 
+void on_infinite_stack_size_clicked (GtkRadioButton *rb, gpointer user_data)
+{
+	prefs.stack_size = RPN_INFINITE_STACK;
+	rpn_stack_set_size (prefs.stack_size);
+}
 /* END */
