@@ -66,8 +66,6 @@ s_active_buttons active_buttons[] = {\
 	{"button_tan", AB_DEC | AB_RPN | AB_PAN}, \
 	{"button_point", ~(AB_BIN | AB_OCT | AB_HEX)}, \
 	{"button_sign", ~(AB_BIN | AB_OCT | AB_HEX)}, \
-	{"button_paropen", ~(AB_RPN)}, \
-	{"button_parclose", ~(AB_RPN)}, \
 	{NULL}\
 };
 
@@ -141,41 +139,43 @@ static void apply_object_data (s_operation_map operation_map[],
 
 static void set_scientific_object_data ()
 {
-	s_operation_map	operation_map[] = {\
-		{"button_pow", '^'},\
-		{"button_lsh", '<'},\
-		{"button_mod", 'm'},\
-		{"button_and", '&'},\
-		{"button_or", '|'},\
-		{"button_xor", 'x'},\
-		{"button_enter", '='},\
-		{"button_plus", '+'},\
-		{"button_minus", '-'},\
-		{"button_mult", '*'},\
-		{"button_div", '/'},\
-		{"button_paropen", '('},\
-		{"button_parclose", ')'},\
-		{"button_percent", '%'},\
-		{NULL}\
+	s_operation_map	operation_map[] = {
+		{"button_pow", '^'},
+		{"button_lsh", '<'},
+		{"button_mod", 'm'},
+		{"button_and", '&'},
+		{"button_or", '|'},
+		{"button_xor", 'x'},
+		{"button_enter", '='},
+		{"button_plus", '+'},
+		{"button_minus", '-'},
+		{"button_mult", '*'},
+		{"button_div", '/'},
+		{"button_percent", '%'},
+		{"button_f1", '('},		/* paropen or swapxy */
+		{"button_f2", ')'},		/* parclose or rolldn */
+		{NULL}
 	};
 	
-	s_gfunc_map gfunc_map[] = {\
-		{"button_sign", display_result_toggle_sign},\
-		{"button_ee", display_append_e},\
-		{NULL}\
+	s_gfunc_map gfunc_map[] = {
+		{"button_sign", display_result_toggle_sign},
+		{"button_ee", display_append_e},
+		{"button_f1", gfunc_f1},	/* paropen or swapxy */
+		{"button_f2", gfunc_f2},	/* parclose or rolldn */
+		{NULL}
 	};
 	
-	s_function_map function_map[] = {\
-		{"button_sin", {sin, asin, sinh, asinh}, TRUE},\
-		{"button_cos", {cos, acos, cosh, acosh}, TRUE},\
-		{"button_tan", {tan, atan, tanh, atanh}, TRUE},\
-		{"button_log", {log10, pow10y, log10, log10}, FALSE},\
-		{"button_ln", {log, exp, log, log}, FALSE},\
-		{"button_sq", {powx2, sqrt, powx2, powx2}, FALSE},\
-		{"button_sqrt", {sqrt, powx2, sqrt, sqrt}, FALSE},\
-		{"button_fac", {factorial, factorial, factorial, factorial}, FALSE},\
-		{"button_cmp", {cmp, cmp, cmp, cmp}, FALSE},\
-		{NULL}\
+	s_function_map function_map[] = {
+		{"button_sin", {sin, asin, sinh, asinh}, TRUE},
+		{"button_cos", {cos, acos, cosh, acosh}, TRUE},
+		{"button_tan", {tan, atan, tanh, atanh}, TRUE},
+		{"button_log", {log10, pow10y, log10, log10}, FALSE},
+		{"button_ln", {log, exp, log, log}, FALSE},
+		{"button_sq", {powx2, sqrt, powx2, powx2}, FALSE},
+		{"button_sqrt", {sqrt, powx2, sqrt, sqrt}, FALSE},
+		{"button_fac", {factorial, factorial, factorial, factorial}, FALSE},
+		{"button_cmp", {cmp, cmp, cmp, cmp}, FALSE},
+		{NULL}
 	};
 
 	apply_object_data (operation_map, gfunc_map, function_map);
@@ -187,26 +187,28 @@ static void set_scientific_object_data ()
 
 static void set_basic_object_data ()
 {
-	s_operation_map	operation_map[] = {\
-		{"button_enter", '='},\
-		{"button_plus", '+'},\
-		{"button_minus", '-'},\
-		{"button_mult", '*'},\
-		{"button_div", '/'},\
-		{"button_paropen", '('},\
-		{"button_parclose", ')'},\
-		{"button_percent", '%'},\
-		{NULL}\
+	s_operation_map	operation_map[] = {
+		{"button_enter", '='},
+		{"button_plus", '+'},
+		{"button_minus", '-'},
+		{"button_mult", '*'},
+		{"button_div", '/'},
+		{"button_percent", '%'},
+		{"button_f1", '('},		/* paropen or swapxy */
+		{"button_f2", ')'},		/* parclose or rolldn */
+		{NULL}
 	};
 	
-	s_gfunc_map gfunc_map[] = {\
-		{"button_sign", display_result_toggle_sign},\
-		{NULL}\
+	s_gfunc_map gfunc_map[] = {
+		{"button_sign", display_result_toggle_sign},
+		{"button_f1", gfunc_f1},	/* paropen or swapxy */
+		{"button_f2", gfunc_f2},	/* parclose or rolldn */
+		{NULL}
 	};
 	
-	s_function_map function_map[] = {\
-		{"button_sqrt", {sqrt, powx2, sqrt, sqrt}, FALSE},\
-		{NULL}\
+	s_function_map function_map[] = {
+		{"button_sqrt", {sqrt, powx2, sqrt, sqrt}, FALSE},
+		{NULL}
 	};
 	
 	apply_object_data (operation_map, gfunc_map, function_map);
@@ -252,9 +254,9 @@ static void ui_pack_from_xml (GtkWidget *box,
 	GtkWidget	*child_widget, *accel_child_widget;
 	GtkAccelGroup	*accel_group;
 	
-	// at first connect signal handlers
+	/* at first connect signal handlers */
 	glade_xml_signal_autoconnect (child_xml);
-	// next, get the "root" child
+	/* next, get the "root" child */
 	child_widget = glade_xml_get_widget (child_xml, child_name);
 	/* we have to add the accel_group of child to the main_window in order
 	 * to get working accelerators.
@@ -295,27 +297,38 @@ void ui_main_window_set_dispctrl (int location)
 {
 	GtkWidget	*table_dispctrl, *box;
 	
-	// destroy any existing display controls
+	/* destroy any existing display controls */
 	if (dispctrl_xml) {
 		table_dispctrl = glade_xml_get_widget (dispctrl_xml, "table_dispctrl");
-		if (table_dispctrl) gtk_widget_destroy (table_dispctrl);
+		if (table_dispctrl) gtk_widget_destroy (table_dispctrl); 
 		g_free (dispctrl_xml);
 	}	
-	// now create the one at location
-	if (location == DISPCTRL_BOTTOM) {
-		box = glade_xml_get_widget (main_window_xml, "window_vbox");
-		dispctrl_xml = glade_file_open (DISPCTRL_BOTTOM_GLADE_FILE, 
-			"table_dispctrl", TRUE);
-		ui_pack_from_xml (box, 2, dispctrl_xml, "table_dispctrl", 
-			"button_clr", TRUE, TRUE);
-	} else if (location == DISPCTRL_RIGHT) {
-		box = glade_xml_get_widget (main_window_xml, "display_hbox");
-		dispctrl_xml = glade_file_open (DISPCTRL_RIGHT_GLADE_FILE, 
-			"table_dispctrl", TRUE);
-		ui_pack_from_xml (box, 1, dispctrl_xml, "table_dispctrl",
-			"button_clr", FALSE, FALSE);
-	} else if (location != DISPCTRL_NONE) 
-		error_message ("Unknown mode in \"ui_main_window_set_dispctrl\"");
+	/* now create the new one at location */
+	switch(location) {
+		case DISPCTRL_BOTTOM:
+			box = glade_xml_get_widget (main_window_xml, "window_vbox");
+			dispctrl_xml = glade_file_open (DISPCTRL_BOTTOM_GLADE_FILE, 
+				"table_dispctrl", TRUE);
+			ui_pack_from_xml (box, 2, dispctrl_xml, "table_dispctrl", 
+				"button_clr", TRUE, TRUE);
+			break;
+		case DISPCTRL_RIGHT:
+			box = glade_xml_get_widget (main_window_xml, "display_hbox");
+			dispctrl_xml = glade_file_open (DISPCTRL_RIGHT_GLADE_FILE, 
+				"table_dispctrl", TRUE);
+			ui_pack_from_xml (box, 1, dispctrl_xml, "table_dispctrl",
+				"button_clr", FALSE, FALSE);
+			break;
+		case DISPCTRL_RIGHTV:
+			box = glade_xml_get_widget (main_window_xml, "display_hbox");
+			dispctrl_xml = glade_file_open (DISPCTRL_RIGHTV_GLADE_FILE, 
+				"table_dispctrl", TRUE);
+			ui_pack_from_xml (box, 1, dispctrl_xml, "table_dispctrl",
+				"button_clr", FALSE, FALSE);
+			break;
+		default:
+			error_message ("Unknown mode in \"ui_main_window_set_dispctrl\"");
+	}
 	set_disp_ctrl_object_data ();
 }
 
@@ -343,7 +356,7 @@ void ui_main_window_buttons_create (int mode)
 {
 	GtkWidget	*box;
 	struct lconv 	*locale_settings;
-
+	
 	if (mode == BASIC_MODE) {
 		button_box_xml = glade_file_open (BASIC_GLADE_FILE, "button_box", TRUE);
 		box = glade_xml_get_widget (main_window_xml, "window_vbox");
@@ -405,7 +418,7 @@ static void set_table_child_font (gpointer data, gpointer user_data)
 	 */
 	if (GTK_IS_BOX (w)) w = ((GtkBoxChild *)((((GtkBox *)w)->children)->data))->widget;
 	if (GTK_IS_LABEL(w)) gtk_widget_modify_font (w, font);
-	// else do nothing
+	/* else do nothing */
 }
 
 /* set_all_buttons_property. calls func with argument data for every button.
@@ -418,7 +431,7 @@ static void set_all_buttons_property (GFunc func, gpointer data)
 	/* at first the display control table. always there; somehor */
 	table = (GtkTable *) glade_xml_get_widget (dispctrl_xml, 
 		"table_dispctrl");
-	// dispctrl_right has an extra table for cosmetic reasons.
+	/* dispctrl_right has an extra table for cosmetic reasons. */
 	if (GTK_IS_TABLE (((GtkTableChild *)table->children->data)->widget))
 		table = (GtkTable *) ((GtkTableChild *)table->children->data)->widget;
 	g_list_foreach (table->children, func, data);
@@ -607,7 +620,7 @@ void position_menu (GtkMenu *menu,
 		children = children->next;
 	}
 	
-	//screen_width = gdk_screen_get_width (gtk_widget_get_screen (widget));
+	/*screen_width = gdk_screen_get_width (gtk_widget_get_screen (widget));*/
 	screen_width = gdk_screen_width ();
 	
 	if (menu_xpos < 0) menu_xpos = 0;
@@ -684,7 +697,7 @@ GtkWidget *ui_pref_dialog_create ()
 	
 	gtk_window_set_title ((GtkWindow *)prefs_dialog, \
 		g_strdup_printf (_("%s Preferences"), PACKAGE));
-	// preferences -> gui
+	/* preferences -> gui */
 	prefs_list = config_file_get_prefs_list();
 	while (prefs_list[counter].key != NULL) {
 		if (prefs_list[counter].set_handler != NULL) {
@@ -704,7 +717,7 @@ GtkWidget *ui_pref_dialog_create ()
 	w = glade_xml_get_widget (prefs_xml, "prefs_bin_length");
 	gtk_widget_set_sensitive (w, prefs.bin_fixed);
 
-	// make user defined constants list.
+	/* make user defined constants list. */
 	
 	store = gtk_list_store_new (NR_CONST_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 	counter = 0;
